@@ -550,6 +550,12 @@ impl<T: Value> Value for Vec<T> {
     }
 }
 
+impl<T: Value> Value for &[T] {
+    fn serialize(&self, buf: &mut Vec<u8>) -> Result<(), ValueTooBig> {
+        serialize_list_or_set(self.iter(), self.len(), buf)
+    }
+}
+
 fn serialize_tuple<V: Value>(
     elem_iter: impl Iterator<Item = V>,
     buf: &mut Vec<u8>,
@@ -568,6 +574,11 @@ fn serialize_tuple<V: Value>(
     Ok(())
 }
 
+fn serialize_empty(buf: &mut Vec<u8>) -> Result<(), ValueTooBig> {
+    buf.put_i32(0);
+    Ok(())
+}
+
 impl Value for CqlValue {
     fn serialize(&self, buf: &mut Vec<u8>) -> Result<(), ValueTooBig> {
         match self {
@@ -602,7 +613,7 @@ impl Value for CqlValue {
             CqlValue::Uuid(u) => u.serialize(buf),
             CqlValue::Varint(v) => v.serialize(buf),
 
-            CqlValue::Empty => Ok(()),
+            CqlValue::Empty => serialize_empty(buf),
         }
     }
 }
@@ -719,7 +730,7 @@ impl_value_list_for_map!(BTreeMap, &str);
 
 // Implement ValueList for tuples of Values of size up to 16
 
-// Here is an example implemetation for (T0, )
+// Here is an example implementation for (T0, )
 // Further variants are done using a macro
 impl<T0: Value> ValueList for (T0,) {
     fn serialized(&self) -> SerializedResult<'_> {
@@ -824,7 +835,7 @@ impl<T: ValueList> BatchValues for Vec<T> {
     }
 }
 
-// Here is an example implemetation for (T0, )
+// Here is an example implementation for (T0, )
 // Further variants are done using a macro
 impl<T0: ValueList> BatchValues for (T0,) {
     fn len(&self) -> usize {
