@@ -918,7 +918,7 @@ impl Connection {
                         params.stream
                     );
                     return Err(QueryError::ProtocolError(
-                        "Received reponse with unexpected StreamId",
+                        "Received response with unexpected StreamId",
                     ));
                 }
                 Orphaned => {
@@ -1094,14 +1094,11 @@ pub async fn open_named_connection(
     let (shard_info, supported_compression, shard_aware_port) = match options_result {
         Response::Supported(mut supported) => {
             let shard_info = ShardInfo::try_from(&supported.options).ok();
-            let supported_compression = supported
-                .options
-                .remove("COMPRESSION")
-                .unwrap_or_else(Vec::new);
+            let supported_compression = supported.options.remove("COMPRESSION").unwrap_or_default();
             let shard_aware_port = supported
                 .options
                 .remove(shard_aware_port_key)
-                .unwrap_or_else(Vec::new)
+                .unwrap_or_default()
                 .into_iter()
                 .next()
                 .and_then(|p| p.parse::<u16>().ok());
@@ -1391,7 +1388,7 @@ impl VerifiedKeyspaceName {
         self.name.as_str()
     }
 
-    // "Keyspace names can have up to 48 alpha-numeric characters and contain underscores;
+    // "Keyspace names can have up to 48 alphanumeric characters and contain underscores;
     // only letters and numbers are supported as the first character."
     // https://docs.datastax.com/en/cql-oss/3.3/cql/cql_reference/cqlCreateKeyspace.html
     // Despite that cassandra accepts underscore as first character so we do too
@@ -1411,7 +1408,7 @@ impl VerifiedKeyspaceName {
             ));
         }
 
-        // Verify all chars are alpha-numeric or underscore
+        // Verify all chars are alphanumeric or underscore
         for character in keyspace_name.chars() {
             match character {
                 'a'..='z' | 'A'..='Z' | '0'..='9' | '_' => {}
