@@ -11,7 +11,7 @@ use std::sync::Arc;
 use std::time::Duration;
 use tokio::net::lookup_host;
 use tokio::time::timeout;
-use tracing::debug;
+use tracing::{debug, trace, trace_span, Instrument};
 use uuid::Uuid;
 
 use super::connection::QueryResponse;
@@ -358,7 +358,7 @@ impl Session {
     ///
     /// This is the easiest way to make a query, but performance is worse than that of prepared queries.
     ///
-    /// See [the book](https://cvybhu.github.io/scyllabook/queries/simple.html) for more information
+    /// See [the book](https://rust-driver.docs.scylladb.com/stable/queries/simple.html) for more information
     /// # Arguments
     /// * `query` - query to perform, can be just a `&str` or the [Query](crate::query::Query) struct.
     /// * `values` - values bound to the query, easiest way is to use a tuple of bound values
@@ -422,6 +422,7 @@ impl Session {
         let query: Query = query.into();
         let serialized_values = values.serialized()?;
 
+        let span = trace_span!("Request", query = query.contents.as_str());
         let response = self
             .run_query(
                 Statement::default(),
@@ -440,6 +441,7 @@ impl Session {
                     }
                 },
             )
+            .instrument(span)
             .await?;
         self.handle_set_keyspace_response(&response).await?;
 
@@ -468,7 +470,7 @@ impl Session {
     /// Returns an async iterator (stream) over all received rows\
     /// Page size can be specified in the [Query](crate::query::Query) passed to the function
     ///
-    /// See [the book](https://cvybhu.github.io/scyllabook/queries/paged.html) for more information
+    /// See [the book](https://rust-driver.docs.scylladb.com/stable/queries/paged.html) for more information
     ///
     /// # Arguments
     /// * `query` - query to perform, can be just a `&str` or the [Query](crate::query::Query) struct.
@@ -508,6 +510,7 @@ impl Session {
             None => self.retry_policy.new_session(),
         };
 
+        let span = trace_span!("Request", query = query.contents.as_str());
         RowIterator::new_for_query(
             query,
             serialized_values.into_owned(),
@@ -517,6 +520,7 @@ impl Session {
             self.cluster.get_data(),
             self.metrics.clone(),
         )
+        .instrument(span)
         .await
     }
 
@@ -530,9 +534,9 @@ impl Session {
     /// > ***Warning***\
     /// > For token/shard aware load balancing to work properly, all partition key values
     /// > must be sent as bound values
-    /// > (see [performance section](https://cvybhu.github.io/scyllabook/queries/prepared.html#performance))
+    /// > (see [performance section](https://rust-driver.docs.scylladb.com/stable/queries/prepared.html#performance))
     ///
-    /// See [the book](https://cvybhu.github.io/scyllabook/queries/prepared.html) for more information
+    /// See [the book](https://rust-driver.docs.scylladb.com/stable/queries/prepared.html) for more information
     ///
     /// # Arguments
     /// * `query` - query to prepare, can be just a `&str` or the [Query](crate::query::Query) struct.
@@ -564,7 +568,7 @@ impl Session {
         let handles = connections.iter().map(|c| c.prepare(&query));
         let mut results = join_all(handles).await;
 
-        // If at least one prepare was succesfull prepare returns Ok
+        // If at least one prepare was successful prepare returns Ok
 
         // Find first result that is Ok, or Err if all failed
         let mut first_ok: Option<Result<PreparedStatement, QueryError>> = None;
@@ -609,9 +613,9 @@ impl Session {
     /// > ***Warning***\
     /// > For token/shard aware load balancing to work properly, all partition key values
     /// > must be sent as bound values
-    /// > (see [performance section](https://cvybhu.github.io/scyllabook/queries/prepared.html#performance))
+    /// > (see [performance section](https://rust-driver.docs.scylladb.com/stable/queries/prepared.html#performance))
     ///
-    /// See [the book](https://cvybhu.github.io/scyllabook/queries/prepared.html) for more information
+    /// See [the book](https://rust-driver.docs.scylladb.com/stable/queries/prepared.html) for more information
     ///
     /// # Arguments
     /// * `prepared` - the prepared statement to execute, generated using [`Session::prepare`](Session::prepare)
@@ -666,6 +670,10 @@ impl Session {
             keyspace: prepared.get_keyspace_name(),
         };
 
+        let span = trace_span!(
+            "Request",
+            prepared_id = format!("{:X}", prepared.get_id()).as_str()
+        );
         let response = self
             .run_query(
                 statement_info,
@@ -677,6 +685,7 @@ impl Session {
                         .await
                 },
             )
+            .instrument(span)
             .await?;
         self.handle_set_keyspace_response(&response).await?;
 
@@ -690,7 +699,7 @@ impl Session {
     /// Page size can be specified in the [PreparedStatement](crate::prepared_statement::PreparedStatement)
     /// passed to the function
     ///
-    /// See [the book](https://cvybhu.github.io/scyllabook/queries/paged.html) for more information
+    /// See [the book](https://rust-driver.docs.scylladb.com/stable/queries/paged.html) for more information
     ///
     /// # Arguments
     /// * `prepared` - the prepared statement to execute, generated using [`Session::prepare`](Session::prepare)
@@ -739,6 +748,10 @@ impl Session {
             None => self.retry_policy.new_session(),
         };
 
+        let span = trace_span!(
+            "Request",
+            prepared_id = format!("{:X}", prepared.get_id()).as_str()
+        );
         RowIterator::new_for_prepared_statement(PreparedIteratorConfig {
             prepared,
             values: serialized_values.into_owned(),
@@ -749,6 +762,7 @@ impl Session {
             cluster_data: self.cluster.get_data(),
             metrics: self.metrics.clone(),
         })
+        .instrument(span)
         .await
     }
 
@@ -758,7 +772,7 @@ impl Session {
     ///
     /// Batch values must contain values for each of the queries
     ///
-    /// See [the book](https://cvybhu.github.io/scyllabook/queries/batch.html) for more information
+    /// See [the book](https://rust-driver.docs.scylladb.com/stable/queries/batch.html) for more information
     ///
     /// # Arguments
     /// * `batch` - [Batch](crate::batch::Batch) to be performed
@@ -805,6 +819,7 @@ impl Session {
             |node: Arc<Node>| async move { node.random_connection().await },
             |connection: Arc<Connection>| async move { connection.batch(batch, values_ref).await },
         )
+        .instrument(trace_span!("Batch"))
         .await
     }
 
@@ -818,12 +833,12 @@ impl Session {
     /// Trying to do two `use_keyspace` requests simultaneously with different names
     /// can end with some connections using one keyspace and the rest using the other.
     ///
-    /// See [the book](https://cvybhu.github.io/scyllabook/queries/usekeyspace.html) for more information
+    /// See [the book](https://rust-driver.docs.scylladb.com/stable/queries/usekeyspace.html) for more information
     ///
     /// # Arguments
     ///
     /// * `keyspace_name` - keyspace name to use,
-    /// keyspace names can have up to 48 alpha-numeric characters and contain underscores
+    /// keyspace names can have up to 48 alphanumeric characters and contain underscores
     /// * `case_sensitive` - if set to true the generated query will put keyspace name in quotes
     /// # Example
     /// ```rust
@@ -884,7 +899,7 @@ impl Session {
 
     /// Get [`TracingInfo`] of a traced query performed earlier
     ///
-    /// See [the book](https://cvybhu.github.io/scyllabook/tracing/tracing.html)
+    /// See [the book](https://rust-driver.docs.scylladb.com/stable/tracing/tracing.html)
     /// for more information about query tracing
     pub async fn get_tracing_info(&self, tracing_id: &Uuid) -> Result<TracingInfo, QueryError> {
         self.get_tracing_info_custom(tracing_id, &GetTracingConfig::default())
@@ -914,7 +929,7 @@ impl Session {
 
         Err(QueryError::ProtocolError(
             "All tracing queries returned an empty result, \
-            maybe information didnt reach this node yet. \
+            maybe information didn't reach this node yet. \
             Consider using get_tracing_info_custom with \
             bigger interval in GetTracingConfig",
         ))
@@ -1038,6 +1053,7 @@ impl Session {
             None => &self.retry_policy,
         };
 
+        #[allow(clippy::unnecessary_lazy_evaluations)]
         let speculative_policy = statement_config
             .speculative_execution_policy
             .as_ref()
@@ -1103,10 +1119,20 @@ impl Session {
         let mut last_error: Option<QueryError> = None;
 
         'nodes_in_plan: for node in query_plan {
+            let span = trace_span!("Executing query", node = node.address.to_string().as_str());
             'same_node_retries: loop {
-                let connection: Arc<Connection> = match choose_connection(node.clone()).await {
+                trace!(parent: &span, "Execution started");
+                let connection: Arc<Connection> = match choose_connection(node.clone())
+                    .instrument(span.clone())
+                    .await
+                {
                     Ok(connection) => connection,
                     Err(e) => {
+                        trace!(
+                            parent: &span,
+                            error = e.to_string().as_str(),
+                            "Choosing connection failed"
+                        );
                         last_error = Some(e);
                         // Broken connection doesn't count as a failed query, don't log in metrics
                         continue 'nodes_in_plan;
@@ -1116,16 +1142,28 @@ impl Session {
                 self.metrics.inc_total_nonpaged_queries();
                 let query_start = std::time::Instant::now();
 
-                let query_result: Result<ResT, QueryError> = do_query(connection).await;
+                trace!(
+                    parent: &span,
+                    connection = connection.get_connect_address().to_string().as_str(),
+                    "Sending"
+                );
+                let query_result: Result<ResT, QueryError> =
+                    do_query(connection).instrument(span.clone()).await;
 
                 last_error = match query_result {
                     Ok(response) => {
+                        trace!(parent: &span, "Query succeeded");
                         let _ = self
                             .metrics
                             .log_query_latency(query_start.elapsed().as_millis() as u64);
                         return Some(Ok(response));
                     }
                     Err(e) => {
+                        trace!(
+                            parent: &span,
+                            last_error = e.to_string().as_str(),
+                            "Query failed"
+                        );
                         self.metrics.inc_failed_nonpaged_queries();
                         Some(e)
                     }
@@ -1140,7 +1178,12 @@ impl Session {
                     ),
                 };
 
-                match retry_session.decide_should_retry(query_info) {
+                let retry_decision = retry_session.decide_should_retry(query_info);
+                trace!(
+                    parent: &span,
+                    retry_decision = format!("{:?}", retry_decision).as_str()
+                );
+                match retry_decision {
                     RetryDecision::RetrySameNode => {
                         self.metrics.inc_retries_num();
                         continue 'same_node_retries;
